@@ -1,6 +1,8 @@
 """Tests for acme.challenges."""
 import unittest
+import warnings
 
+import josepy as jose
 import mock
 import OpenSSL
 import requests
@@ -8,11 +10,8 @@ import requests
 from six.moves.urllib import parse as urllib_parse  # pylint: disable=import-error
 
 from acme import errors
-from acme import jose
-from acme import other
 from acme import test_util
 
-
 CERT = test_util.load_comparable_cert('cert.pem')
 KEY = jose.JWKRSA(key=test_util.load_rsa_private_key('rsa512_key.pem'))
 
@@ -78,6 +77,77 @@ class KeyAuthorizationChallengeResponseTest(unittest.TestCase):
         self.assertFalse(response.verify(self.chall, KEY.public_key()))
 
 
+class DNS01ResponseTest(unittest.TestCase):
+    # pylint: disable=too-many-instance-attributes
+
+    def setUp(self):
+        from acme.challenges import DNS01Response
+        self.msg = DNS01Response(key_authorization=u'foo')
+        self.jmsg = {
+            'resource': 'challenge',
+            'type': 'dns-01',
+            'keyAuthorization': u'foo',
+        }
+
+        from acme.challenges import DNS01
+        self.chall = DNS01(token=(b'x' * 16))
+        self.response = self.chall.response(KEY)
+
+    def test_to_partial_json(self):
+        self.assertEqual(self.jmsg, self.msg.to_partial_json())
+
+    def test_from_json(self):
+        from acme.challenges import DNS01Response
+        self.assertEqual(self.msg, DNS01Response.from_json(self.jmsg))
+
+    def test_from_json_hashable(self):
+        from acme.challenges import DNS01Response
+        hash(DNS01Response.from_json(self.jmsg))
+
+    def test_simple_verify_failure(self):
+        key2 = jose.JWKRSA.load(test_util.load_vector('rsa256_key.pem'))
+        public_key = key2.public_key()
+        verified = self.response.simple_verify(self.chall, "local", public_key)
+        self.assertFalse(verified)
+
+    def test_simple_verify_success(self):
+        public_key = KEY.public_key()
+        verified = self.response.simple_verify(self.chall, "local", public_key)
+        self.assertTrue(verified)
+
+
+class DNS01Test(unittest.TestCase):
+
+    def setUp(self):
+        from acme.challenges import DNS01
+        self.msg = DNS01(token=jose.decode_b64jose(
+            'evaGxfADs6pSRb2LAv9IZf17Dt3juxGJ+PCt92wr+oA'))
+        self.jmsg = {
+            'type': 'dns-01',
+            'token': 'evaGxfADs6pSRb2LAv9IZf17Dt3juxGJ-PCt92wr-oA',
+        }
+
+    def test_validation_domain_name(self):
+        self.assertEqual('_acme-challenge.www.example.com',
+                         self.msg.validation_domain_name('www.example.com'))
+
+    def test_validation(self):
+        self.assertEqual(
+            "rAa7iIg4K2y63fvUhCfy8dP1Xl7wEhmQq0oChTcE3Zk",
+            self.msg.validation(KEY))
+
+    def test_to_partial_json(self):
+        self.assertEqual(self.jmsg, self.msg.to_partial_json())
+
+    def test_from_json(self):
+        from acme.challenges import DNS01
+        self.assertEqual(self.msg, DNS01.from_json(self.jmsg))
+
+    def test_from_json_hashable(self):
+        from acme.challenges import DNS01
+        hash(DNS01.from_json(self.jmsg))
+
+
 class HTTP01ResponseTest(unittest.TestCase):
     # pylint: disable=too-many-instance-attributes
 
@@ -291,20 +361,29 @@ class TLSSNI01ResponseTest(unittest.TestCase):
 class TLSSNI01Test(unittest.TestCase):
 
     def setUp(self):
-        from acme.challenges import TLSSNI01
-        self.msg = TLSSNI01(
-            token=jose.b64decode('a82d5ff8ef740d12881f6d3c2277ab2e'))
         self.jmsg = {
             'type': 'tls-sni-01',
             'token': 'a82d5ff8ef740d12881f6d3c2277ab2e',
         }
 
+    def _msg(self):
+        from acme.challenges import TLSSNI01
+        with warnings.catch_warnings(record=True) as warn:
+            warnings.simplefilter("always")
+            msg = TLSSNI01(
+                token=jose.b64decode('a82d5ff8ef740d12881f6d3c2277ab2e'))
+            assert warn is not None # using a raw assert for mypy
+            self.assertTrue(len(warn) == 1)
+            self.assertTrue(issubclass(warn[-1].category, DeprecationWarning))
+            self.assertTrue('deprecated' in str(warn[-1].message))
+        return msg
+
     def test_to_partial_json(self):
-        self.assertEqual(self.jmsg, self.msg.to_partial_json())
+        self.assertEqual(self.jmsg, self._msg().to_partial_json())
 
     def test_from_json(self):
         from acme.challenges import TLSSNI01
-        self.assertEqual(self.msg, TLSSNI01.from_json(self.jmsg))
+        self.assertEqual(self._msg(), TLSSNI01.from_json(self.jmsg))
 
     def test_from_json_hashable(self):
         from acme.challenges import TLSSNI01
@@ -319,236 +398,68 @@ class TLSSNI01Test(unittest.TestCase):
     @mock.patch('acme.challenges.TLSSNI01Response.gen_cert')
     def test_validation(self, mock_gen_cert):
         mock_gen_cert.return_value = ('cert', 'key')
-        self.assertEqual(('cert', 'key'), self.msg.validation(
+        self.assertEqual(('cert', 'key'), self._msg().validation(
             KEY, cert_key=mock.sentinel.cert_key))
         mock_gen_cert.assert_called_once_with(key=mock.sentinel.cert_key)
 
-
-class RecoveryContactTest(unittest.TestCase):
+class TLSALPN01ResponseTest(unittest.TestCase):
+    # pylint: disable=too-many-instance-attributes
 
     def setUp(self):
-        from acme.challenges import RecoveryContact
-        self.msg = RecoveryContact(
-            activation_url='https://example.ca/sendrecovery/a5bd99383fb0',
-            success_url='https://example.ca/confirmrecovery/bb1b9928932',
-            contact='c********n@example.com')
+        from acme.challenges import TLSALPN01Response
+        self.msg = TLSALPN01Response(key_authorization=u'foo')
         self.jmsg = {
-            'type': 'recoveryContact',
-            'activationURL': 'https://example.ca/sendrecovery/a5bd99383fb0',
-            'successURL': 'https://example.ca/confirmrecovery/bb1b9928932',
-            'contact': 'c********n@example.com',
+            'resource': 'challenge',
+            'type': 'tls-alpn-01',
+            'keyAuthorization': u'foo',
+        }
+
+        from acme.challenges import TLSALPN01
+        self.chall = TLSALPN01(token=(b'x' * 16))
+        self.response = self.chall.response(KEY)
+
+    def test_to_partial_json(self):
+        self.assertEqual(self.jmsg, self.msg.to_partial_json())
+
+    def test_from_json(self):
+        from acme.challenges import TLSALPN01Response
+        self.assertEqual(self.msg, TLSALPN01Response.from_json(self.jmsg))
+
+    def test_from_json_hashable(self):
+        from acme.challenges import TLSALPN01Response
+        hash(TLSALPN01Response.from_json(self.jmsg))
+
+
+class TLSALPN01Test(unittest.TestCase):
+
+    def setUp(self):
+        from acme.challenges import TLSALPN01
+        self.msg = TLSALPN01(
+            token=jose.b64decode('a82d5ff8ef740d12881f6d3c2277ab2e'))
+        self.jmsg = {
+            'type': 'tls-alpn-01',
+            'token': 'a82d5ff8ef740d12881f6d3c2277ab2e',
         }
 
     def test_to_partial_json(self):
         self.assertEqual(self.jmsg, self.msg.to_partial_json())
 
     def test_from_json(self):
-        from acme.challenges import RecoveryContact
-        self.assertEqual(self.msg, RecoveryContact.from_json(self.jmsg))
+        from acme.challenges import TLSALPN01
+        self.assertEqual(self.msg, TLSALPN01.from_json(self.jmsg))
 
     def test_from_json_hashable(self):
-        from acme.challenges import RecoveryContact
-        hash(RecoveryContact.from_json(self.jmsg))
+        from acme.challenges import TLSALPN01
+        hash(TLSALPN01.from_json(self.jmsg))
 
-    def test_json_without_optionals(self):
-        del self.jmsg['activationURL']
-        del self.jmsg['successURL']
-        del self.jmsg['contact']
+    def test_from_json_invalid_token_length(self):
+        from acme.challenges import TLSALPN01
+        self.jmsg['token'] = jose.encode_b64jose(b'abcd')
+        self.assertRaises(
+            jose.DeserializationError, TLSALPN01.from_json, self.jmsg)
 
-        from acme.challenges import RecoveryContact
-        msg = RecoveryContact.from_json(self.jmsg)
-
-        self.assertTrue(msg.activation_url is None)
-        self.assertTrue(msg.success_url is None)
-        self.assertTrue(msg.contact is None)
-        self.assertEqual(self.jmsg, msg.to_partial_json())
-
-
-class RecoveryContactResponseTest(unittest.TestCase):
-
-    def setUp(self):
-        from acme.challenges import RecoveryContactResponse
-        self.msg = RecoveryContactResponse(token='23029d88d9e123e')
-        self.jmsg = {
-            'resource': 'challenge',
-            'type': 'recoveryContact',
-            'token': '23029d88d9e123e',
-        }
-
-    def test_to_partial_json(self):
-        self.assertEqual(self.jmsg, self.msg.to_partial_json())
-
-    def test_from_json(self):
-        from acme.challenges import RecoveryContactResponse
-        self.assertEqual(
-            self.msg, RecoveryContactResponse.from_json(self.jmsg))
-
-    def test_from_json_hashable(self):
-        from acme.challenges import RecoveryContactResponse
-        hash(RecoveryContactResponse.from_json(self.jmsg))
-
-    def test_json_without_optionals(self):
-        del self.jmsg['token']
-
-        from acme.challenges import RecoveryContactResponse
-        msg = RecoveryContactResponse.from_json(self.jmsg)
-
-        self.assertTrue(msg.token is None)
-        self.assertEqual(self.jmsg, msg.to_partial_json())
-
-
-class ProofOfPossessionHintsTest(unittest.TestCase):
-
-    def setUp(self):
-        jwk = KEY.public_key()
-        issuers = (
-            'C=US, O=SuperT LLC, CN=SuperTrustworthy Public CA',
-            'O=LessTrustworthy CA Inc, CN=LessTrustworthy But StillSecure',
-        )
-        cert_fingerprints = (
-            '93416768eb85e33adc4277f4c9acd63e7418fcfe',
-            '16d95b7b63f1972b980b14c20291f3c0d1855d95',
-            '48b46570d9fc6358108af43ad1649484def0debf',
-        )
-        subject_key_identifiers = ('d0083162dcc4c8a23ecb8aecbd86120e56fd24e5')
-        authorized_for = ('www.example.com', 'example.net')
-        serial_numbers = (34234239832, 23993939911, 17)
-
-        from acme.challenges import ProofOfPossession
-        self.msg = ProofOfPossession.Hints(
-            jwk=jwk, issuers=issuers, cert_fingerprints=cert_fingerprints,
-            certs=(CERT,), subject_key_identifiers=subject_key_identifiers,
-            authorized_for=authorized_for, serial_numbers=serial_numbers)
-
-        self.jmsg_to = {
-            'jwk': jwk,
-            'certFingerprints': cert_fingerprints,
-            'certs': (jose.encode_b64jose(OpenSSL.crypto.dump_certificate(
-                OpenSSL.crypto.FILETYPE_ASN1, CERT.wrapped)),),
-            'subjectKeyIdentifiers': subject_key_identifiers,
-            'serialNumbers': serial_numbers,
-            'issuers': issuers,
-            'authorizedFor': authorized_for,
-        }
-        self.jmsg_from = self.jmsg_to.copy()
-        self.jmsg_from.update({'jwk': jwk.to_json()})
-
-    def test_to_partial_json(self):
-        self.assertEqual(self.jmsg_to, self.msg.to_partial_json())
-
-    def test_from_json(self):
-        from acme.challenges import ProofOfPossession
-        self.assertEqual(
-            self.msg, ProofOfPossession.Hints.from_json(self.jmsg_from))
-
-    def test_from_json_hashable(self):
-        from acme.challenges import ProofOfPossession
-        hash(ProofOfPossession.Hints.from_json(self.jmsg_from))
-
-    def test_json_without_optionals(self):
-        for optional in ['certFingerprints', 'certs', 'subjectKeyIdentifiers',
-                         'serialNumbers', 'issuers', 'authorizedFor']:
-            del self.jmsg_from[optional]
-            del self.jmsg_to[optional]
-
-        from acme.challenges import ProofOfPossession
-        msg = ProofOfPossession.Hints.from_json(self.jmsg_from)
-
-        self.assertEqual(msg.cert_fingerprints, ())
-        self.assertEqual(msg.certs, ())
-        self.assertEqual(msg.subject_key_identifiers, ())
-        self.assertEqual(msg.serial_numbers, ())
-        self.assertEqual(msg.issuers, ())
-        self.assertEqual(msg.authorized_for, ())
-
-        self.assertEqual(self.jmsg_to, msg.to_partial_json())
-
-
-class ProofOfPossessionTest(unittest.TestCase):
-
-    def setUp(self):
-        from acme.challenges import ProofOfPossession
-        hints = ProofOfPossession.Hints(
-            jwk=KEY.public_key(), cert_fingerprints=(),
-            certs=(), serial_numbers=(), subject_key_identifiers=(),
-            issuers=(), authorized_for=())
-        self.msg = ProofOfPossession(
-            alg=jose.RS256, hints=hints,
-            nonce=b'xD\xf9\xb9\xdbU\xed\xaa\x17\xf1y|\x81\x88\x99 ')
-
-        self.jmsg_to = {
-            'type': 'proofOfPossession',
-            'alg': jose.RS256,
-            'nonce': 'eET5udtV7aoX8Xl8gYiZIA',
-            'hints': hints,
-        }
-        self.jmsg_from = {
-            'type': 'proofOfPossession',
-            'alg': jose.RS256.to_json(),
-            'nonce': 'eET5udtV7aoX8Xl8gYiZIA',
-            'hints': hints.to_json(),
-        }
-
-    def test_to_partial_json(self):
-        self.assertEqual(self.jmsg_to, self.msg.to_partial_json())
-
-    def test_from_json(self):
-        from acme.challenges import ProofOfPossession
-        self.assertEqual(
-            self.msg, ProofOfPossession.from_json(self.jmsg_from))
-
-    def test_from_json_hashable(self):
-        from acme.challenges import ProofOfPossession
-        hash(ProofOfPossession.from_json(self.jmsg_from))
-
-
-class ProofOfPossessionResponseTest(unittest.TestCase):
-
-    def setUp(self):
-        # acme-spec uses a confusing example in which both signature
-        # nonce and challenge nonce are the same, don't make the same
-        # mistake here...
-        signature = other.Signature(
-            alg=jose.RS256, jwk=KEY.public_key(),
-            sig=b'\xa7\xc1\xe7\xe82o\xbc\xcd\xd0\x1e\x010#Z|\xaf\x15\x83'
-                b'\x94\x8f#\x9b\nQo(\x80\x15,\x08\xfcz\x1d\xfd\xfd.\xaap'
-                b'\xfa\x06\xd1\xa2f\x8d8X2>%d\xbd%\xe1T\xdd\xaa0\x18\xde'
-                b'\x99\x08\xf0\x0e{',
-            nonce=b'\x99\xc7Q\xb3f2\xbc\xdci\xfe\xd6\x98k\xc67\xdf',
-        )
-
-        from acme.challenges import ProofOfPossessionResponse
-        self.msg = ProofOfPossessionResponse(
-            nonce=b'xD\xf9\xb9\xdbU\xed\xaa\x17\xf1y|\x81\x88\x99 ',
-            signature=signature)
-
-        self.jmsg_to = {
-            'resource': 'challenge',
-            'type': 'proofOfPossession',
-            'nonce': 'eET5udtV7aoX8Xl8gYiZIA',
-            'signature': signature,
-        }
-        self.jmsg_from = {
-            'resource': 'challenge',
-            'type': 'proofOfPossession',
-            'nonce': 'eET5udtV7aoX8Xl8gYiZIA',
-            'signature': signature.to_json(),
-        }
-
-    def test_verify(self):
-        self.assertTrue(self.msg.verify())
-
-    def test_to_partial_json(self):
-        self.assertEqual(self.jmsg_to, self.msg.to_partial_json())
-
-    def test_from_json(self):
-        from acme.challenges import ProofOfPossessionResponse
-        self.assertEqual(
-            self.msg, ProofOfPossessionResponse.from_json(self.jmsg_from))
-
-    def test_from_json_hashable(self):
-        from acme.challenges import ProofOfPossessionResponse
-        hash(ProofOfPossessionResponse.from_json(self.jmsg_from))
+    def test_validation(self):
+        self.assertRaises(NotImplementedError, self.msg.validation, KEY)
 
 
 class DNSTest(unittest.TestCase):
